@@ -15,6 +15,7 @@ export class Component {
   constructor(args) {
     this.id = args.id;
     this.name = args.name;
+    this.key = args.key;
     this.messageUrl = args.messageUrl;
     this.csrfTokenHeaderName = args.csrfTokenHeaderName;
 
@@ -28,6 +29,7 @@ export class Component {
 
     this.document = args.document || document;
     this.walker = args.walker || walk;
+    this.window = args.window || window;
 
     this.root = undefined;
     this.modelEls = [];
@@ -35,6 +37,7 @@ export class Component {
     this.loadingEls = [];
     this.keyEls = [];
     this.errors = {};
+    this.return = {};
     this.poll = {};
 
     this.actionQueue = [];
