@@ -12,7 +12,7 @@ import orjson
 from .call_method_parser import InvalidKwarg, parse_call_method_name, parse_kwarg
 from .components import UnicornField, UnicornView
 from .errors import UnicornViewError
-from .utils import generate_checksum
+from .message import ComponentRequest, Return
 
 
 logger = logging.getLogger(__name__)
@@ -159,10 +159,9 @@ def _get_property_value(component: UnicornView, property_name: str) -> Any:
 
 def _call_method_name(
     component: UnicornView, method_name: str, params: List[Any]
-) -> None:
+) -> Any:
     """
     Calls the method name with parameters.
-    Also updates the data dictionary which gets set back as part of the payload.
 
     Args:
         param component: Component to call method on.
@@ -174,47 +173,9 @@ def _call_method_name(
         func = getattr(component, method_name)
 
         if params:
-            func(*params)
+            return func(*params)
         else:
-            func()
-
-
-class ComponentRequest:
-    """
-    Parses, validates, and stores all of the data from the message request.
-    """
-
-    def __init__(self, request):
-        self.body = {}
-
-        try:
-            self.body = orjson.loads(request.body)
-            assert self.body, "Invalid JSON body"
-        except orjson.JSONDecodeError as e:
-            raise UnicornViewError("Body could not be parsed") from e
-
-        self.data = self.body.get("data")
-        assert self.data is not None, "Missing data"  # data could theoretically be {}
-
-        self.id = self.body.get("id")
-        assert self.id, "Missing component id"
-
-        self.validate_checksum()
-
-        self.action_queue = self.body.get("actionQueue", [])
-
-    def validate_checksum(self):
-        """
-        Validates that the checksum in the request matches the data.
-
-        Returns:
-            Raises `AssertionError` if the checksums don't match.
-        """
-        checksum = self.body.get("checksum")
-        assert checksum, "Missing checksum"
-
-        generated_checksum = generate_checksum(orjson.dumps(self.data))
-        assert checksum == generated_checksum, "Checksum does not match"
+            return func()
 
 
 @handle_error
@@ -257,6 +218,7 @@ def message(request: HttpRequest, component_name: str = None) -> JsonResponse:
     component.hydrate()
 
     is_reset_called = False
+    return_data = None
 
     for action in component_request.action_queue:
         action_type = action.get("type")
@@ -324,6 +286,7 @@ def message(request: HttpRequest, component_name: str = None) -> JsonResponse:
             assert call_method_name, "Missing 'name' key for callMethod"
 
             (method_name, params) = parse_call_method_name(call_method_name)
+            return_data = Return(method_name, params)
             setter_method = {}
 
             if "=" in call_method_name:
@@ -339,9 +302,8 @@ def message(request: HttpRequest, component_name: str = None) -> JsonResponse:
                 property_value = setter_method[property_name]
 
                 _set_property_value(component, property_name, property_value)
+                return_data = Return(property_name, [property_value])
             else:
-                (method_name, params) = parse_call_method_name(call_method_name)
-
                 if method_name == "$refresh":
                     # Handle the refresh special action
                     component = UnicornView.create(
@@ -373,7 +335,9 @@ def message(request: HttpRequest, component_name: str = None) -> JsonResponse:
                     validate_all_fields = True
                 else:
                     component.calling(method_name, params)
-                    _call_method_name(component, method_name, params)
+                    return_data.value = _call_method_name(
+                        component, method_name, params
+                    )
                     component.called(method_name, params)
         else:
             raise UnicornViewError(f"Unknown action_type '{action_type}'")
@@ -402,4 +366,9 @@ def message(request: HttpRequest, component_name: str = None) -> JsonResponse:
         "errors": component.errors,
     }
 
+    if return_data:
+        res.update(
+            {"redirect": return_data.redirect, "return": return_data.get_data(),}
+        )
+
     return JsonResponse(res)
